from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
import numpy as np
import pandas as pd
from sklearn.decomposition import PCA

class pcaSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal()


class pcaProcess(QRunnable):

    def __init__(self,parameters,nameExp):
        super(pcaProcess, self).__init__()
        self.dfs = parameters
        self.nameExp = nameExp
        self.signals = pcaSignals()

    @pyqtSlot()
    def run(self):

        try:

            arrayDF=[]
            infoDF =[]
            strainDF = []
            repDF = []
            dfPuntosPCA=pd.DataFrame(columns=["PCA_1", "PCA_2", "Strain", "Repetition"])

            for df in self.dfs:

                arrayDF.append(df['Entropy'])
                infoDF.append(df['info'])

                info_df=str(df['info'].iloc[0])
                strain_df=info_df.split(',')[0]
                repet=info_df.split(',')[1]
                strainDF.append(strain_df)
                repDF.append(repet)



            X = np.squeeze(np.array([arrayDF]))
            infoDF = np.squeeze(np.array([infoDF]))
            pca = PCA(n_components=2)
            auxReturn = pca.fit_transform(X)
            pcaVector = pca.components_

            dfPuntosPCA['PCA_1']=auxReturn[:,0]
            dfPuntosPCA['PCA_2']=auxReturn[:,1]
            dfPuntosPCA['Strain']=strainDF
            dfPuntosPCA['Repetition']=repDF



        except Exception as e:
            print (e)
            self.signals.error.emit()

        finally:

            self.signals.finished.emit([dfPuntosPCA,pcaVector,infoDF, self.nameExp,self.dfs])  # Done



import pandas as pd

def codonsToAminos(df):
    
    df_a = pd.DataFrame(columns=['Position','K','N','T','R','S','I','M','Q','H', 'P','L','E','D','A','G','V','*','Y','C','W','F','Passage','Strain','Repetition'])
#    df_a = df.copy()
    df_a['Position'] = df.Position
    df_a['Passage'] = df.Passage
    df_a['Strain'] = df.Strain
    df_a['Repetition'] = df.Repetition
    df_a['K'] = df.AAA + df.AAG
    df_a['N'] = df.AAC + df.AAT
    df_a['T'] = df.ACA + df.ACC +df.ACG + df.ACT
    df_a['R'] = df.AGA + df.AGG + df.CGA + df.CGC + df.CGG + df.CGT
    df_a['S'] = df.AGC + df.AGT + df.TCA + df.TCC + df.TCG + df.TCT
    df_a['I'] = df.ATA + df.ATC +df.ATT
    df_a['M'] = df.ATG
    df_a['Q'] = df.CAA + df.CAG
    df_a['H'] = df.CAC + df.CAT
    df_a['P'] = df.CCA + df.CCC + df.CCG + df.CCT
    df_a['L'] = df.CTA + df.CTC + df.CTG + df.CTT + df.TTA + df.TTG
    df_a['E'] = df.GAA + df.GAG
    df_a['D'] = df.GAC + df.GAT
    df_a['A'] = df.GCA + df.GCC + df.GCG + df.GCT
    df_a['G'] = df.GGA + df.GGC + df.GGG + df.GGT
    df_a['V'] = df.GTA + df.GTC + df.GTG + df.GTT
    df_a['*'] = df.TAA + df.TAG + df.TGA
    df_a['Y'] = df.TAC + df.TAT
    df_a['C'] = df.TGC + df.TGT
    df_a['W'] = df.TGG
    df_a['F'] = df.TTC + df.TTT
    return df_a
    





import numpy as np
import pandas as pd
def experimentToPassages(df):
    #Función que toma el experimento (dataframe df) y convierte cada pasaje en una fila
    
    #Busco los pasajes
    passages=df['Passage'].unique()
    
    #Obtengo la info
    info_df=str(df['info'].iloc[0])
    strain_df=info_df.split(',')[0]
    repetition_df=info_df.split(',')[1]
    
  
    arrayDF_P  = []
    arrayInfo = []
    arrayStrain = []

    for p in passages:
#        dfP=df[df.Passage.eq(p)]
#        dfP_val=dfP[dfP.Position.isin(posVal)].copy()
#        passageArray=np.array(dfP_val[['Entropy']])

        dfP=df[df.Passage.eq(p)]
#        print(isinstance(dfP,pd.DataFrame))
        dfPE=dfP[['Entropy']]
#        print(isinstance(dfPE,pd.DataFrame))
        passageArray=np.array(dfPE)
        
        pA = np.squeeze(np.array([passageArray]))
        arrayDF_P.append(pA)
        arrayInfo.append(str(strain_df)+','+str(repetition_df)+','+str(p))
        arrayStrain.append(str(strain_df))
    return np.array(arrayDF_P), np.array(arrayInfo), np.array(arrayStrain)

        
def totalExperimentsToPassages(parameters):
    #Función que toma todos los experimentos que haya cargados en parameters y convierte cada experimento a una fila por pasaje
    
    #Arrays donde se guardan los pasajes como filas y la info
    arrayT_DF_P  = []
    arrayT_Info = []
    arrayT_Strain = []
    
    totalArrayData = []
    totalArrayInfo = []
    totalArrayStrain = []
#    firstIteration = True
    
    for df in parameters:
        arrayP, arrayI, arrayS=experimentToPassages(df)
        arrayT_DF_P.append(arrayP)
        arrayT_Info.append(arrayI)
        arrayT_Strain.append(arrayS)
    
    totalArrayData = np.concatenate(arrayT_DF_P)
    totalArrayInfo = np.concatenate(arrayT_Info)
    totalArrayStrain = np.concatenate(arrayT_Strain)
        
    return totalArrayData, totalArrayInfo, totalArrayStrain

def experimentToPositions(df):
    #Función que toma el experimento (dataframe df) y convierte cada pasaje en una fila
    
    #Busco los pasajes
    positions=df['Position'].unique()
    
    #Obtengo la info
    info_df=str(df['info'].iloc[0])
    strain_df=info_df.split(',')[0]
    repetition_df=info_df.split(',')[1]
    
    arrayDF_P  = []
    arrayInfo = []
    arrayStrain = []
    
    for p in positions:
        dfP=df[df.Position.eq(p)]
#        print(isinstance(dfP,pd.DataFrame))
        dfPE=dfP[['Entropy']]
#        print(isinstance(dfPE,pd.DataFrame))
        positionArray=np.array(dfPE)
        
        pA = np.squeeze(np.array([positionArray]))
        arrayDF_P.append(pA)
        arrayInfo.append(str(strain_df)+','+str(repetition_df)+','+str(p))
        arrayStrain.append(str(strain_df))
    return np.array(arrayDF_P), np.array(arrayInfo), np.array(arrayStrain)

def totalExperimentsToPositions(parameters):
    #Función que toma todos los experimentos que haya cargados en parameters y convierte cada experimento a una fila por pasaje
    
    #Arrays donde se guardan las posiciones como filas y la info
    arrayT_DF_P  = []
    arrayT_Info = []
    arrayT_Strain = []
    
    totalArrayData = []
    totalArrayInfo = []
    totalArrayStrain = []
#    firstIteration = True
    
    for df in parameters:
        arrayP, arrayI, arrayS=experimentToPositions(df)
        arrayT_DF_P.append(arrayP)
        arrayT_Info.append(arrayI)
        arrayT_Strain.append(arrayS)
    
    totalArrayData = np.concatenate(arrayT_DF_P)
    totalArrayInfo = np.concatenate(arrayT_Info)
    totalArrayStrain = np.concatenate(arrayT_Strain)
        
    return totalArrayData, totalArrayInfo, totalArrayStrain

def formatStrains(strainT):
    #Función que toma el array de strains y le asigna a cada una un int a partir del 0
    #Ademas calcula el valor que debe tomar el parámetro perplexity de tSNE de acuerdo a la cantidad de puntos de cada cepa
    
    #Paso a dataframe las cepas de los experimentos cargados
    df = pd.DataFrame(strainT,columns=["Strain"])
    df = df.assign(StrainNumber = df['Strain'])
    #Busco la cantidad de pasajes de cada cepa
    strainsU=df.Strain.unique()
    dataRows = []
    i=0
    for s in strainsU:
        n=df[df.Strain.eq(s)].count()
        dataRows.append(n)
        df.loc[df['Strain'] == s, 'StrainNumber'] = i
#        df[df.Strain.eq(s)]["StrainNumber"]=i
        i+=1
    arNum = np.array(dataRows)
    num = arNum.min()
#    perp = p//3
#    print(perp)
#    print(df.shape)
    return num, df


import pandas as pd
import numpy as np
import math
import os
import time
path_pkl = 'src/data_experiments/'


def getEntropyExperiment(strain,repetition,fvp,lvp,dataFrame):
    filePath=path_pkl +str(strain)+'_' +str(repetition)+'('+str(fvp)+'-'+str(lvp)+')_entropy.parquet'

    if os.path.isfile(filePath):
        return pd.read_parquet(filePath)
    else:
        entropyList = []
        epsilon = 2.2204e-16
        for index, row in dataFrame.iterrows():
            i = 0
            entropy = float(0)
            for freq in row:
                i += 1
                if i > 2 and i < 67: # i=1 está pasaje, i=2 está coverage y apartir de i=3 tenes los 64 codones
                    entropy -= np.multiply(freq, (math.log(freq + epsilon)))  # logaritmo en base e de la frequencia
            entropyList.append(entropy)

        dataFrame['Entropy'] = np.asarray(entropyList)
        df = dataFrame[['Position','Coverage','Strain','Repetition','Passage','Entropy','info']]

        return df
    

def getEntropyExperimentAminos(strain,repetition,fvp,lvp,dataFrame):
    filePath=path_pkl +str(strain)+'_' +str(repetition)+'('+str(fvp)+'-'+str(lvp)+')_aminos_entropy.parquet'

    if os.path.isfile(filePath):
        return pd.read_parquet(filePath)
    else:

        entropyList = []
        epsilon = 2.2204e-16
        for index, row in dataFrame.iterrows():
            i = 0
            entropy = float(0)
            for freq in row:
                i += 1
                if i > 1 and i < 23: # i=1 está pasaje, i=2 está coverage y apartir de i=3 tenes los 64 codones
                    entropy -= np.multiply(freq, (math.log(freq + epsilon)))  # logaritmo en base e de la frequencia
            entropyList.append(entropy)

        dataFrame['Entropy'] = np.asarray(entropyList)
        dataFrame.to_parquet(filePath)
        return dataFrame



import matplotlib.pyplot as plt

def plotEntropyPosition(position, dataFrame,strain,repetition,color = 'red'):
    passage=dataFrame.loc[dataFrame.Position==position,'Passage']
    entropy=dataFrame.loc[dataFrame.Position==position,'Entropy']
    fig1, ax = plt.subplots()
    ax.plot(passage, entropy,color=color)
    ax.set_xlabel('Passages')
    ax.set_ylabel('Entropy')
    ax.set_title("Entropy over passages for position "+str(position))
    plt.grid()
    name = 'src/plot_images/Entropy_' + str(position) +'_'+strain+'_'+repetition+ '.png'
    plt.savefig(name)
    plt.close()


def plotEntropiesPosition(position, dataFrames):
    fig1, ax = plt.subplots()
    for df in dataFrames:
        passage=df.loc[df.Position==position,'Passage']
        entropy = df.loc[df.Position == position, 'Entropy']
        ax.plot(passage, entropy)
    ax.set_xlabel('Passages')
    ax.set_ylabel('Entropy')
    ax.set_title("Entropy over passages for position " + str(position))
    plt.grid()
    name = 'src/plot_images/Entropy_' + str(position) +'.png'
    plt.savefig(name)
    plt.close()

    
    
    



from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
from src.TSNE_exp import tsnePassages,tsnePositions

class tsneSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal()


class tsneProcess(QRunnable):

    def __init__(self,parameters,tsneType,nameExp):
        super(tsneProcess, self).__init__()
        self.data = parameters
        self.type = tsneType
        self.signals = tsneSignals()
        self.nameExp = nameExp


    @pyqtSlot()
    def run(self):

        try:
            if self.type == 'Passages':
                df = tsnePassages(self.data)
            else:
                df = tsnePositions(self.data)


        except Exception as e:
            print (e)
            self.signals.error.emit()

        finally:
            self.signals.finished.emit([self.data,df,self.type,self.nameExp])  # Done


class tsneProcessUpdate(QRunnable):
#dfs,perp,lr,data,"Passages",nameExps
    def __init__(self,parameters,perp,lr,oldData,tsneType,nameExp):
        super(tsneProcessUpdate, self).__init__()
        self.data = parameters
        self.type = type
        self.signals = tsneSignals()
        self.lr = lr
        self.perp = perp
        self.oldData = oldData
        self.type = tsneType
        self.nameExp = nameExp


    @pyqtSlot()
    def run(self):
        try:
            if self.type == 'Passages':
                df = tsnePassages(self.data,[self.perp],[self.lr],self.oldData)
            else:
                df = tsnePositions(self.data, [self.perp], [self.lr], self.oldData)


        except Exception as e:
            print(e)
            self.signals.error.emit()

        finally:
            self.signals.finished.emit([df,self.perp,self.lr,self.type,self.nameExp])  # Done


from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot

from src.ReadData import readData,keepValidPositions
from src.GetEntropy import getEntropyExperiment, getEntropyExperimentAminos
import os
import pandas as pd



path_pkl = 'src/data_experiments/' #también a cambiar

class loadSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal(object)


class LoadFolder(QRunnable):

    def __init__(self,filename,fvp,lvp):
        super(LoadFolder, self).__init__()
        self.filename = filename
        self.fvp = fvp
        self.lvp = lvp
        self.signals = loadSignals()


    @pyqtSlot()
    def run(self):

        try:

            fileName = self.filename + '/'
            parameters = [fileName, self.fvp, self.lvp]
            df,strain,repetition,self.lvp = readData(parameters)

            experimentName = strain + '_' + repetition + '(' + str(self.fvp) + '-' + str(self.lvp) + ').parquet'
            entropyName = strain + '_' + repetition + '(' + str(self.fvp) + '-' + str(self.lvp) + ')_entropy.parquet'
            df.to_parquet(path_pkl + experimentName)
            dfAux = getEntropyExperiment(strain, repetition, self.fvp, self.lvp, df)
            dfAux.to_parquet(path_pkl + entropyName)


        except Exception as e:
            self.signals.error.emit(e)

        finally:
            self.signals.finished.emit([strain, repetition, self.fvp, self.lvp, True])  # Done

class LoadFile(QRunnable):

    def __init__(self,filename):
        super(LoadFile, self).__init__()
        self.filename = filename
        self.signals = loadSignals()
        self.fvp = 0
        self.lvp = 0
    @pyqtSlot()
    def run(self):

        try:
            fileName = self.filename
            auxSplit = fileName.split('/')
            cantidad = len(auxSplit)
            print(cantidad)
            i = 0
            directorio = ''
            while i < (cantidad - 1):
                if i == 0:
                    directorio = auxSplit[i]
                else:
                    directorio = str(directorio) + '/' + str(auxSplit[i])
                i = i + 1
            directorio = str(directorio) + '/'
            archivo = auxSplit[cantidad - 1]

            aux = str(os.getcwd()) + '/' + path_pkl
            archivoAux = archivo.split('.')[0]
            strain_repetion = archivoAux.split('_')
            strain = strain_repetion[0]
            auxSplit = strain_repetion[1].split('(')
            repetition = auxSplit[0]
            aux_fvp_lvp = auxSplit[1].split('-')
            self.fvp = int(aux_fvp_lvp[0])
            self.lvp = int(aux_fvp_lvp[1].split(')')[0])
            if "entropy" not in archivo:
                aux_name_entroy = archivoAux + '_entropy.parquet'
                print(aux_name_entroy)
                if os.path.exists(directorio + aux_name_entroy):
                    if aux != directorio:
                        df = pd.read_parquet(directorio + archivo)
                        df.to_parquet(path_pkl + archivo)
                        df = pd.read_parquet(directorio + aux_name_entroy)
                        df.to_parquet(path_pkl + aux_name_entroy)
                else:
                    if aux != directorio:
                        df = pd.read_parquet(directorio + archivo)
                        df.to_parquet(path_pkl + archivo)
                        df = getEntropyExperiment(strain, repetition, self.fvp, self.lvp, df)
                        df.to_parquet(path_pkl + aux_name_entroy)
                    else:
                        df = pd.read_parquet(directorio + archivo)
                        df = getEntropyExperiment(os.getcwd(), strain, repetition, self.fvp, self.lvp, df)
                        df.to_parquet(path_pkl + aux_name_entroy)
            else:
                if aux != directorio:
                    auxExperiment = archivo.replace('_entropy.parquet', '.parquet')
                    if os.path.exists(directorio + auxExperiment):
                        df = pd.read_parquet(directorio + auxExperiment)
                        df.to_parquet(path_pkl + auxExperiment)
                    df = pd.read_parquet(directorio + archivo)
                    df.to_parquet(path_pkl + archivo)


        except Exception as e:
            print(e)
            self.signals.error.emit()

        finally:
            self.signals.finished.emit([strain, repetition, self.fvp, self.lvp, False])  # Done



import pandas as pd
import numpy as np
import os
import glob
from src.NumericalSort import numericalSort
import re

#format name_passage_strain_repetition.csv
path_pkl = 'src/data_experiments/'
def readData(parameters):
    i = 0
    strainControl = ''
    repetitionControl = ''
    path=parameters[0]
    fvp=parameters[1]
    lvp=parameters[2]

    df_list = []
    for filename in sorted(glob.glob(os.path.join(path, '*.csv')) , key=numericalSort):
        auxInfo = filename.split('/')
        auxInfo = auxInfo[len(auxInfo)-1]
        auxInfo = auxInfo.split('_')
        auxPassage = auxInfo[1]
        strain = auxInfo[2]
        auxRepetition = auxInfo[3]
        repetition = re.sub("\D+", '', auxRepetition)
        passage = re.sub("\D+",'',auxPassage)
        if i == 0: #to check only one type of strain and repetition
            strainControl = strain
            repetitionControl = repetition
            i = i +1
        if repetition != repetitionControl or strainControl != strain:
            raise Exception("Error, there are csv files from other experiment")
        df=pd.read_csv(filename)
        df['Passage']=passage
        df_list.append(df)



    df_T = pd.concat(df_list)
    df_T['Strain']=strain
    df_T['Repetition']=repetition
    df_T['info'] = df_T.apply(lambda row: str(row.Strain) + ',' + str(row.Repetition) + "," + str(row.Passage) + "," + str(row.Position),axis=1)

    if lvp == 0:
        positions = df_T['Position'].unique()
        lvp = positions[len(positions) - 1]
    df_T =keepValidPositions(df_T,fvp,lvp)


    return df_T, strain, repetition, lvp


def keepValidPositions(dfE, fvp, lvp):
    # Función que toma el dataframe y se queda solamente con las posiciones válidas

    # Busco el valor de las posiciones en el dataframe
    posiciones = dfE.Position.unique()

    # Encuentro los indices donde las posiciones son validas
    idx = np.where((posiciones >= fvp) & (posiciones <= lvp))

    # Guardo en posVal las posiciones validas
    posVal = posiciones[idx]

    # Me quedo con las posiciones validas del dataframe
    dfE_val = dfE[dfE.Position.isin(posVal)].copy()

    return dfE_val



import pandas as pd
import math
import matplotlib.pyplot as plt
import scipy


def dgumbel(x,mu,s): #density function
    return math.exp((mu - x)/s - math.exp((mu - x)/s))/s

def pgumbel(q,mu,s): #distribution function
    return math.exp(-(math.exp(-((q - mu)/s))))

def qgumbel(p,mu,s): #Quantile function
    return (mu - s) * math.log(-math.log(p))

def gumbelPlot(df,strain,repetition):
    dataframe= extraerMaximos(df)
    gumbelAdjust(dataframe,strain,repetition)

def extraerMaximos(df):
    df_positions =df.Position.unique()

    entropies = []
    i=0
    for position in df_positions:
        entropy=df.loc[df.Position==position,'Entropy'].max(axis=0)
        entropies.append(entropy)
        i=i+1
    df = pd.DataFrame(entropies)

    return df

def get_hist(ax):
    n,bins = [],[]
    for rect in ax.patches:
        ((x0, y0), (x1, y1)) = rect.get_bbox().get_points()
        n.append(y1-y0)
        bins.append(x0) # left edge of each bin
    bins.append(x1) # also get right edge of last bin

    return n,bins




def gumbelAdjust(dfMax,strain,repetition):
    #fig, ax = plt.subplots()

    sub = dfMax.quantile(.95, axis = 0)
    quantil= sub.iloc[0]
    listDF = dfMax.values.tolist()
    subsample=[]
    for value in listDF:
        if value[0] <= quantil:
            subsample.append(value)
    dfHist = pd.DataFrame(subsample)
    dfHist.columns=['Entropy']
    axsub=dfHist['Entropy'].hist(bins=20,grid=False, xlabelsize=12, ylabelsize=12, density=True)
    n, bins = get_hist(axsub)


    plt.xlabel("Entropy", fontsize=15)
    plt.ylabel("Frequency", fontsize=15)
    plt.xlim([0.0, 0.2])
    mu, sigma = scipy.stats.norm.fit(subsample)

    x=[]

    y=[]
    for point in bins:
        x.append(point)
        a=dgumbel(point,mu,sigma)
        y.append(a)
        #plt.plot(a,x,'bo')
        #aux=aux+sum
    plt.plot(x,y, 'r-', label='density gumbel', linewidth=1)
    name = 'src/plot_images/Gumbel'+'_'+strain+'_'+repetition + '.png'
    plt.savefig(name)
    plt.close()






from PyQt5 import QtWidgets,uic
from src.dialogError import dError

class valuesPositions(QtWidgets.QDialog):
    def __init__(self, parent=None):
        super(valuesPositions, self).__init__()
        uic.loadUi('src/ui_files/dialogPositions.ui', self)
        self.pushButton.clicked.connect(self.setParameters)
        self.fvpValue=774
        self.lvpValue=7332
    def setParameters(self):

        fvp = self.FVP.toPlainText()
        lvp = self.LVP.toPlainText()
        try:
            if fvp == '':
                fvp = 0
            if lvp == '':
                lvp = 0
            self.fvpValue = int(fvp)
            self.lvpValue = int(lvp)
            self.close()
        except:
            de = dError("Error: Enter only integers")
            de.exec()

    def getFVP(self):
        return self.fvpValue
    def getLVP(self):
        return self.lvpValue


import numpy as np
import pandas as pd
from PIL import Image
from sklearn.decomposition import PCA
import matplotlib

matplotlib.use('Qt5Agg')
from PyQt5 import QtCore, QtWidgets, uic
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from src.PlotEntropyPosition import plotEntropyPosition, plotEntropiesPosition


class FigurePCA(FigureCanvas):

    def __init__(self, dfPuntosPCA, pcaVector, infoDF, nameExps,dfs, parent, width=5, height=4, dpi=100):

        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        FigureCanvas.__init__(self, fig)
        self.parent = parent

        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

        self.dfPuntosPCA = dfPuntosPCA
        self.infoDF = infoDF
        self.pcaVector = pcaVector
        self.nameExps = nameExps
        self.dfsAux = dfs
        self.fig = fig
        self.aux = []
        self.compute_initial_figure()
        self.name = ''
        self._flag = False

    def updatePCA(self,dfPuntosPCA,pcaVector,infoDF,nameExps,dfs):
        self.dfPuntosPCA = dfPuntosPCA
        self.pcaVector = pcaVector
        self.infoDF = infoDF
        self.nameExps = nameExps
        self.dfsAux = dfs
        self.axes.cla()
        self.compute_initial_figure()

    def compute_initial_figure(self):

        strains = self.dfPuntosPCA.Strain.unique()

        self.fig.canvas.mpl_connect('button_press_event', self.check_click)
        self.aux = self.axes.scatter(self.pcaVector[0, :], self.pcaVector[1, :], marker='o', c="grey", alpha=0.2)
        for s in strains:
            d = self.dfPuntosPCA.loc[self.dfPuntosPCA['Strain'] == s]
            x = d.PCA_1 / 60
            y = d.PCA_2 / 60
            self.axes.scatter(x, y, label=s)
        self.axes.set_xlabel('X1')
        self.axes.set_ylabel('X2')
        self.axes.legend()
        title = 'Distribucion coeficientes de componentes principales posicion '
        plt.suptitle(title, fontsize=16)
        self.axes.grid(True)
        self.draw()

    def find_nearest(self, points, value):
        arrayAux = np.asarray(points)
        idx = (np.abs(arrayAux - value)).argmin()
        return idx

    def check_click(self, event):
        if event.inaxes == self.axes:
            cont, ind = self.aux.contains(event)
            if cont:
                self.onclick(event)


    def onclick(self, event):
        ix, iy = event.xdata, event.ydata
        indx = self.find_nearest(self.pcaVector[0,], ix)
        exp = self.infoDF[0,]
        experiment = exp[indx]
        experimentAux = experiment.split(',')
        position = experimentAux[3]
        plotEntropiesPosition(int(position), self.dfsAux)
        self.name = 'src/plot_images/Entropy_' + position + '.png'
        img = Image.open(self.name)
        titleAux = "plot_images/Entropy for position " + position
        img.show(title=titleAux)



import numpy as np
import matplotlib
matplotlib.use('Qt5Agg')
from PyQt5 import QtWidgets
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
import pandas as pd

dfPlot = ''

class FigureTSNE(FigureCanvas):

    def __init__(self,dataframes,data, tsneType,nameExp, parent,lr=None,perp=None,width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        FigureCanvas.__init__(self, fig)
        self.parent = parent
        FigureCanvas.setSizePolicy(self,
                                   QtWidgets.QSizePolicy.Expanding,
                                   QtWidgets.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)
        self.dfs = dataframes
        if dataframes == None:
            self.isImport = True
        else:
            self.isImport = False
        self.data = data
        self.fig = fig
        self.type = tsneType
        self.array = []
        if lr ==None:
            self.perp=3
            self.lr = 10
        else:
            self.perp=perp
            self.lr=lr
        self.nameExp = nameExp
        self.removeStrains=[]
        self.compute_initial_figure()
        self.name = ''
        self._flag = False
        self.normal = True # to difference between normal tsne execution and add lr and perp

    def getData(self):
        if self.isImport:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: This is an imported experiment. Please recalculate the experiment as normal')
            error_dialog.exec()
        else:
            return self.dfs,self.data,self.nameExp

    def getType(self):
        return self.type

    def newtSNE(self,dataframes,data, tsneType,nameExp,lr=None,perp=None):
        self.removeStrains = []
        if dataframes == None:
            self.isImport = True
        else:
            self.isImport = False
        if lr==None:
            self.perp = 3
            self.lr = 10
        else:
            self.perp=perp
            self.lr = lr
        self.normal = True
        self.data=data
        self.dfs= dataframes
        self.axes.cla()
        self.perp=3
        self.lr = 10
        self.type = tsneType
        self.nameExp = nameExp
        self.compute_initial_figure()

    def updateData(self,newData,perp,lr,expName):
        if self.isImport:
            error_dialog = QtWidgets.QErrorMessage()
            error_dialog.showMessage('Error: This is an imported experiment. Please recalculate the experiment as normal')
            error_dialog.exec()
        else:
            self.normal = False
            self.data = newData
            self.lr = lr
            self.perp = perp
            self.expName = expName
            self.updateParameters(perp,lr)

    def updateParameters(self,perp,lr):

        self.axes.cla()
        self.perp = perp
        self.lr = lr
        self.compute_initial_figure()

    def exportCSV(self,filename):
        df = self.data
        df = df.drop(columns=['StrainNumber','Plot'])
        if self.type == 'Positions':
            df["Position"] = df.apply(lambda row: row.Info.split(",")[2], axis=1)
            columnsTitles = ['Position', 'Strain', 'Repetition', 'tsne_1', 'tsne_2', 'LR', 'Perp', 'Info']
        else:
            df["Passage"] = df.apply(lambda row: row.Info.split(",")[2], axis=1)
            columnsTitles = ['Passage', 'Strain', 'Repetition', 'tsne_1', 'tsne_2', 'LR', 'Perp', 'Info']
        df["Repetition"] = df.apply(lambda row: row.Info.split(",")[1], axis=1)

        df = df.reindex(columns=columnsTitles)

        exps = ''
        i = 0
        j = 0

        while i < len(self.nameExp):
            j +=1
            if j==len(self.nameExp):
                exps += self.nameExp[i].split('.')[0]
            else:
                exps += self.nameExp[i].split('.')[0]+'-'
            i+=1
        if not self.normal:
            df = df[df.Perp.eq(self.perp) & df.LR.eq(self.lr)]

        df.to_csv(filename)
        return self.nameExp, self.normal,self.lr,self.perp,self.type




    def compute_initial_figure(self):

        self.array=[]
        global dfPlot
        dfPlot = self.data[self.data.Perp.eq(self.perp) & self.data.LR.eq(self.lr)]


        #fig, ax = plt.subplots()
        self.annot = self.axes.annotate("", xy=(0, 0), xytext=(20, 20), textcoords="offset points",
                            bbox=dict(boxstyle="round", fc="w"),
                            arrowprops=dict(arrowstyle="->"))

        self.annot.set_visible(False)
        self.fig.canvas.mpl_connect("motion_notify_event", self.hover)

        strains = dfPlot.Strain.unique()
        #print (str(strains))
        indexColor = 0
        for s in strains:
            if s not in self.removeStrains:
                d = dfPlot.loc[dfPlot['Strain'] == s]
                x = d.tsne_1
                y = d.tsne_2
                aux =self.axes.scatter(x, y, label=s, color='C'+str(indexColor), alpha=0.3)
                self.array.append(aux)
            indexColor = indexColor + 1
        self.axes.legend()
        self.draw()


    def onOffStrains(self,strains):
        self.removeStrains=strains
        self.axes.cla()
        self.compute_initial_figure()


    def update_annot(self,ind,i):
        pos = self.array[i].get_offsets()[ind["ind"][0]]
        self.annot.xy = pos
        global dfPlot
        dinfo =dfPlot.loc[dfPlot.tsne_1.eq(pos[0]) & dfPlot.tsne_2.eq(pos[1])]
        if len(dinfo['Info'].values) > 0:
            info = dinfo['Info'].values[0]
            auxInfo= info.split(',')
            if self.type == 'Passages':
                pointInfo='Repetition: ' + auxInfo[1] + '\nPassage: ' + auxInfo[2]
            else:
                pointInfo = 'Repetition: ' + auxInfo[1] + '\nPositions: ' + auxInfo[2]

            self.annot.set_text(pointInfo)
            self.annot.get_bbox_patch().set_alpha(0.4)

    def hover(self,event):
        vis = self.annot.get_visible()
        if event.inaxes == self.axes:
            cont = False
            i = 0
            while not cont and i < len(self.array):
                cont, ind = self.array[i].contains(event)
                i = i +1
            if cont:
                self.update_annot(ind,(i-1))
                self.annot.set_visible(True)
                self.fig.canvas.draw_idle()
            else:
                if vis:
                    self.annot.set_visible(False)
                    self.fig.canvas.draw_idle()





from PyQt5 import QtWidgets,uic

path_buttons='src/buttons_and_background/'
class dOK(QtWidgets.QDialog):
    def __init__(self, errorMessage,parent=None):
        super(dOK, self).__init__()
        uic.loadUi('src/ui_files/dialogOK.ui', self)
        self._flag = False
        self.pushButton.clicked.connect(self.closeWindow)
        self.pushButton.setStyleSheet("QPushButton{border-image:url("+path_buttons+"Ok.png);}")
        self.pushButton.setToolTip("Click to close window")
        self.text.setText(errorMessage)
    def closeWindow(self):
        self.close()



from setuptools import setup, find_packages
from io import open
from os import path
import pathlib
# The directory containing this file
HERE = pathlib.Path(__file__).parent
# The text of the README file
README = (HERE / "README.md").read_text()
# automatically captured required modules for install_requires in requirements.txt and as well as configure dependency links
install_requires = [
    'matplotlib==3.2.1',
    'numpy==1.18.4',
    'pandas==0.25.3',
    'Pillow==7.1.2',
    'scikit-learn==0.22.2.post1',
    'scipy==1.4.1',
    'pyqt5==5.15',
    'pyqtchart==5.15',
    'dnspython==1.16.0',
    'fastparquet==0.4.0',
    'pymongo==3.11.0',
    'pyarrow==0.16.0'
    ]

setup (
 name = 'vev-fing',
 description = 'Visualization and Evolution of ARN Virus',
 version = '1.0.9',
 packages = find_packages(), # list of all packages
 install_requires = install_requires,
 python_requires='>=3.6', # any python greater than 3.6
 entry_points={
        'console_scripts': [
            'vev = src.vev:main',
        ],
    },
 include_package_data=True,
 author="Federico Aicardi,Rodrigo Cespedes",
 keyword="ARN,VIRUS,Evolution,Analysis,Visualization",
 long_description=README,
 long_description_content_type="text/markdown",
 license='MIT',
 url='', 
  author_email='vev-fing@gmail.com',
  classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ]
)



import numpy as np
import matplotlib.pyplot as plt

def plotPopularDecay(df):
    #Busco el valor del primer pasaje
    minPassage=df['Passage'].min()
    
    #Busco los pasajes
    passages=df['Passage'].unique()
    
    #Obtengo la info
    info_df=str(df['info'].iloc[0])
    strain_df=info_df.split(',')[0]
    repetition_df=info_df.split(',')[1]
    
    #Busco el valor de las posiciones en el dataframe
    posVal=df.Position.unique()

    
    #Busco los valores de las probs. para el primer pasaje
    dfFirstPassage=df[df.Passage.eq(minPassage)]
    dfFirstPassage_val=dfFirstPassage[dfFirstPassage.Position.isin(posVal)].copy()
    dfFirstPassage_val_aux=dfFirstPassage_val.drop(['Position','Coverage','Passage','Strain','Repetition','info'], axis=1)
    maxColumnsCodon=dfFirstPassage_val_aux.idxmax(axis=1)
    dfFirstPassage_val['MaxFreqCodon']=maxColumnsCodon
    dfFirstPassage_val=dfFirstPassage_val[['Position','MaxFreqCodon']]
    return dfFirstPassage_val
#    pos=366
#    maxF=dfFirstPassage_val.loc[dfFirstPassage_val['Position'] == pos].MaxFreq.astype(str)
#    print('Codon: ',maxF)
#    freqEvol=df.loc[df['Position']==pos]#.columns.str.startswith('ATT')
#    freqEvol=freqEvol[maxF.astype(str)]
    plt.figure(1)
#    curve=[]
    for pos in dfFirstPassage_val['Position']:
        maxF_Cod=dfFirstPassage_val.loc[dfFirstPassage_val['Position'] == pos].MaxFreqCodon.astype(str)
        freqEvol=df.loc[df['Position']==pos]
        freqEvol=freqEvol[maxF_Cod.astype(str)]
        freqEvol=np.array(freqEvol)
        if (np.amin(freqEvol)<th):
            str_label=str(pos)+' - '+str(maxF_Cod.values.item())
#            print(maxF_Cod)
            plt.plot(passages,freqEvol, label=str_label)
            plt.legend(loc='best',borderaxespad=0., title='Posición - Codón')
#            print(str(pos))
#            print(str(maxF_Cod))
#            plt.text(freqEvol[0],np.amin(freqEvol),str_label)
#            item=[pos,maxF,freqEvol]
#            print('Minima frecuencia: ',np.amin(freqEvol))
#            curve.append(item)
    
        
#        print('Position: ',pos)
#        print('Codon: ',maxF)
    plt.xlabel('Passages')
    plt.ylabel('Frequencies')

        
#    curveArray=np.array(curve)
#    return curveArray


    
    
    



from PyQt5.QtCore import QObject,QRunnable,pyqtSignal,pyqtSlot
import pandas as pd
from src.GetEntropySurface import getEntropySurface
from src.PlotEntropyPosition import plotEntropyPosition
from src.calculateGumbel import gumbelPlot
from src.PlotPopularDecay import plotPopularDecay
path_pkl = 'src/data_experiments/'

class plotSignals(QObject):

    finished = pyqtSignal(list)
    error = pyqtSignal(object)

class plotSurface(QRunnable):

    def __init__(self,strain,repetition,fvp,lvp):
        super(plotSurface, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()
    @pyqtSlot()
    def run(self):
        try:
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'
            df = pd.read_parquet(path_pkl + experimentName)
            getEntropySurface(df, self.strain, self.repetition)
        except Exception as e:
            self.signals.error.emit(e)
        finally:
            self.signals.finished.emit([self.strain,self.repetition,self.fvp,self.lvp])


class plotPositionEntropy(QRunnable):

    def __init__(self,strain,repetition,position,fvp,lvp):
        super(plotPositionEntropy, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.position = position
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            intposition = int(self.position)
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'
            df = pd.read_parquet(path_pkl + experimentName)
            plotEntropyPosition(intposition, df, self.strain, self.repetition, 'red')
        except:
            self.signals.error.emit("Please Select an Integer Position")
        finally:
            self.signals.finished.emit([self.strain,self.repetition,self.position,self.fvp,self.lvp])

class plotGumbel(QRunnable):
    def __init__(self,strain,repetition,fvp,lvp):
        super(plotGumbel, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ')_entropy.parquet'
            df = pd.read_parquet(path_pkl + experimentName)
            gumbelPlot(df,self.strain,self.repetition)
        except:
            self.signals.error.emit("ERROR")
        finally:
            self.signals.finished.emit([self.strain,self.repetition,self.fvp,self.lvp])


class codonDecay(QRunnable):
    def __init__(self,strain,repetition,th,fvp,lvp):
        super(codonDecay, self).__init__()
        self.strain = strain
        self.repetition = repetition
        self.th = th
        self.fvp = fvp
        self.lvp = lvp
        self.signals = plotSignals()

    @pyqtSlot()
    def run(self):
        try:
            experimentName = self.strain + '_' + self.repetition + '(' + self.fvp + '-' + self.lvp + ').parquet'
            df = pd.read_parquet(path_pkl + experimentName)
            dfReturn=plotPopularDecay(df,)
        except:
            self.signals.error.emit("ERROR")
        finally:
            self.signals.finished.emit([dfReturn,df,self.th,self.strain,self.repetition,self.fvp,self.lvp])


import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from scipy.ndimage.filters import uniform_filter1d
from src.Format_DF import totalExperimentsToPassages, formatStrains, experimentToPositions



def filtradoPromedioMovil_DF(df,vent):

    """
    Función que toma el datafame de un experimento completo y le hace el promedio móvil
    del tamaño indicado por vent
    """

    df_FPM=df.copy(deep=True)
    df_FPM['EntrPromMov']=None
#    df_FPM=df_FPM.astype({'Passage':int})

    passages=df['Passage'].unique()
    positions=df['Position'].unique()

    indMax=passages.size


    for pos in positions:
        for pas in passages:
            '''Encuentro el índice dentro del array passages del elemento pass'''
            result = np.where(passages == pas)

#            print(result[0])

            sumandos=0
            promediados=0
            sumandos+=1

            for i in range(int(np.floor(vent/2))):
                if ((result[0]-(i+1))>=0):
                    sumandos+=1
#                    print('Posicion: '+str(pos)+', pasaje: '+str(pas)+', valor de i por el -: '+str(i+1))
                    indice = result[0] - (i+1)
                    pass_i=passages[indice]
                    valor = df_FPM.loc[(df_FPM['Passage'] == pass_i[0]) & (df_FPM['Position'] == pos), 'Entropy']
#                    print(valor)
                    promediados = promediados + valor
                if((result[0]+(i+1))<indMax):
                    sumandos+=1
#                    print('Posicion: '+str(pos)+', pasaje: '+str(pas)+', valor de i por el +: '+str(i+1))
                    indice = result[0] - (i+1)
                    pass_i=passages[indice]
                    valor = df_FPM.loc[(df_FPM['Passage'] == pass_i[0]) & (df_FPM['Position'] == pos), 'Entropy']
#                    print(valor)
                    promediados = promediados + valor
            ent_PM=promediados/sumandos
            df_FPM.loc[(df_FPM['Passage'] == pass_i[0]) & (df_FPM['Position'] == pos), 'EntrPromMov'] = ent_PM



    return df_FPM


#def movingAverage_1(arr,n):
#
#    ret = np.cumsum(arr, axis=0, dtype=float)
##    return ret
#    ret[n:] -= ret[:-n]
#    return ret[n - 1:] / n
#
#def movingAverage_2(arr, n=3):
#
#    ma=np.zeros(arr.shape)
#
#    for i in range(arr.shape[0]):
#        ma[i,:]=np.convolve(arr[i,:], np.ones(arr.shape[1]), mode='valid')
#
#    return ma


def movingAverage_1D(arr, n=3):

    y_f = uniform_filter1d(arr, mode='constant', size=n)

    r=int(np.floor(n/2)+1)

#    print('El principio')

    for i in range(r-1):
        d=((np.floor(n/2)+1)+i)
#        print(d)
        y_f[i]=n*y_f[i]/d

#    print('El final')

    for i in range(y_f.shape[0]-r+1,y_f.shape[0]):
        d=y_f.shape[0]-i+np.floor(n/2)
#        print(d)
        y_f[i]=n*y_f[i]/d


    return y_f


def movingAverage(arr, n=3):

    arr_MA = np.zeros_like(arr)

    for i in range(arr.shape[0]):
        arr_MA[i]=movingAverage_1D(arr[i],n)
    return arr_MA


def filtradoPromedioMovil(df, ventana):

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    passages=df['Passage'].unique()

    arr_Low = movingAverage(ar_Data, n=ventana)
    arr_High=np.abs(ar_Data-arr_Low)

    df_Data=pd.DataFrame(ar_Data)
    df_Data.columns=[passages]
#    df_Data['Position']=positions
    df_D = pd.concat([df_Data, df_I], axis=1)

    df_Data_Low=pd.DataFrame(arr_Low)
    df_Data_Low.columns=[passages]
#    df_Data_Low['Position']=positions
    df_D_L = pd.concat([df_Data_Low, df_I], axis=1)

    df_Data_High=pd.DataFrame(arr_High)
    df_Data_High.columns=[passages]
#    df_Data_High['Position']=positions
    df_D_H = pd.concat([df_Data_High, df_I], axis=1)

    return df_D, df_D_L, df_D_H


def descomposicion_PM_LV_RV(df, ventana):

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    arr_Low = movingAverage(ar_Data, n=ventana)
    arr_High=np.abs(ar_Data-arr_Low)

    ar_Data_Sum=np.sum(ar_Data, axis=1)
    arr_Low_Sum=np.sum(arr_Low, axis=1)
    arr_HighSum=np.sum(arr_High, axis=1)

    data_Aux=pd.DataFrame(columns=["Info", "Position", "Strain", "Repetition", "Entropy_Acum", "Entropy_Low", "Entropy_High"])

    data_Aux['Info']=ar_Info
    data_Aux['Position']=df_I.Position
    data_Aux['Strain']=df_I.Strain
    data_Aux['Repetition']=df_I.Repetition
    data_Aux['Entropy_Acum']=ar_Data_Sum
    data_Aux['Entropy_Low']=arr_Low_Sum
    data_Aux['Entropy_High']=arr_HighSum

    return data_Aux


def descomposicion_PM_LV_RV_Experiments(experiments, ventana):

    l_LV_RV=[]

    for df in experiments:

        descomposicion = descomposicion_PM_LV_RV(df, ventana)
        l_LV_RV.append(descomposicion)

    return l_LV_RV


def filtradoPoly(df, grado):

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    passages=df['Passage'].unique()

    passages = list(map(int, passages))

    arr_Low = np.zeros_like(ar_Data)

    for i in range(ar_Data.shape[0]):
        z = np.polyfit(passages, ar_Data[i], grado)
        p = np.poly1d(z)
        arr_Low[i]=p(passages)

    arr_High=np.abs(ar_Data-arr_Low)

    df_Data=pd.DataFrame(ar_Data)
    df_Data.columns=[passages]
    df_D = pd.concat([df_Data, df_I], axis=1)

    df_Data_Low=pd.DataFrame(arr_Low)
    df_Data_Low.columns=[passages]
    df_Data_Low = pd.concat([df_Data_Low, df_I], axis=1)

    df_Data_High=pd.DataFrame(arr_High)
    df_Data_High.columns=[passages]
    df_Data_High = pd.concat([df_Data_High, df_I], axis=1)

    return df_D, df_Data_Low, df_Data_High


def descomposicion_Poly_LV_RV(df, grado):

    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)

    df_I = pd.DataFrame([sub.split(",") for sub in ar_Info])
    df_I.columns=["Strain", "Repetition", "Position"]

    passages=df['Passage'].unique()

    passages = list(map(int, passages))

    arr_Low = np.zeros_like(ar_Data)

    for i in range(ar_Data.shape[0]):
        z = np.polyfit(passages, ar_Data[i], grado)
        p = np.poly1d(z)
        arr_Low[i]=p(passages)

    arr_High=np.abs(ar_Data-arr_Low)

    ar_Data_Sum=np.sum(ar_Data, axis=1)
    arr_Low_Sum=np.sum(arr_Low, axis=1)
    arr_HighSum=np.sum(arr_High, axis=1)

    data_Aux=pd.DataFrame(columns=["Info", "Position", "Strain", "Repetition", "Entropy_Acum", "Entropy_Low", "Entropy_High"])

    data_Aux['Info']=ar_Info
    data_Aux['Position']=df_I.Position
    data_Aux['Strain']=df_I.Strain
    data_Aux['Repetition']=df_I.Repetition
    data_Aux['Entropy_Acum']=ar_Data_Sum
    data_Aux['Entropy_Low']=arr_Low_Sum
    data_Aux['Entropy_High']=arr_HighSum

    return data_Aux


def descomposicion_Poly_LV_RV_Experiments(experiments, grado):

    l_LV_RV=[]

    for df in experiments:

        descomposicion = descomposicion_Poly_LV_RV(df, grado)
        l_LV_RV.append(descomposicion)

    return l_LV_RV




#def polynomialFilter(df):
#
#    ar_Data, ar_Info, ar_Strain = experimentToPositions(df)
#
#    passages = df['Passage'].unique()
#    passages = list(map(int, passages))
#
#    f = interpolate.interp1d(passages, ar_Data, kind='cubic')
#
#    return f

def plot_LV_RV(exps):

    fig, ax = plt.subplots()
    for df in exps:
        info=(df.loc[0].Info).split(",")
        x=df.Entropy_Low
        y=df.Entropy_High
#        ax.scatter(x, y, alpha=0.5)
        ax.scatter(x, y, alpha=0.5, label="Cepa {s}, repetición {r}".format(s=info[0], r=info[1]))
    plt.grid()
    plt.legend(bbox_to_anchor=(1.1, 1.05))
    return fig



import pandas as pd
import numpy as np

#para pasajes iniciales y ultimos ver casos

def interpolationPassageFreq(passage, dataframe):
    passages = dataframe.Passage.unique().tolist()
    if not passage in passages:
        passage = str(int(passage) + 1)
    i = passages.index(passage)
    auxPassagePlus = passages[i + 1]
    auxPassageMinus = passages[i - 1]
    columns = dataframe.columns
    i=0
    for c in columns:
       if i > 1 and i < 66: #all the codons
           dataframe.loc[dataframe.Passage == passage, c] = None
           df1 = dataframe.loc[dataframe.Passage == auxPassageMinus, c].tolist()
           df3 = dataframe.loc[dataframe.Passage == auxPassagePlus, c].tolist()
           df2 = dataframe.loc[dataframe.Passage == passage, c].tolist()
           dfs = [df1, df2, df3]
           dfAux = pd.DataFrame(dfs)
           dfAux = dfAux.interpolate()
           aux = dfAux.iloc[[1]].values.tolist()
           dataframe.loc[dataframe.Passage == passage, c] = np.asarray(aux[0])
       i=i+1



    return dataframe


def interpolationPassage(passage, dataframe):
    passages = dataframe.Passage.unique().tolist()
    if not passage in passages:
        passage = str(int(passage) + 1)
    i = passages.index(passage)
    auxPassagePlus = passages[i + 1]
    auxPassageMinus = passages[i - 1]
    dataframe.loc[dataframe.Passage == passage, 'Entropy'] = None
    df1 = dataframe.loc[dataframe.Passage == auxPassageMinus, 'Entropy'].tolist()
    df3 = dataframe.loc[dataframe.Passage == auxPassagePlus, 'Entropy'].tolist()
    df2 = dataframe.loc[dataframe.Passage == passage, 'Entropy'].tolist()
    dfs = [df1, df2, df3]
    dfAux = pd.DataFrame(dfs)
    dfAux = dfAux.interpolate()
    aux = dfAux.iloc[[1]].values.tolist()
    dataframe.loc[dataframe.Passage == passage, 'Entropy'] = np.asarray(aux[0])

    return dataframe
